<?php

namespace App\Exceptions;

use App\Helpers\General;
use Exception;
use Illuminate\Foundation\Exceptions\Handler as ExceptionHandler;
use Symfony\Component\HttpKernel\Exception\MethodNotAllowedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Throwable;

class Handler extends ExceptionHandler
{
    /**
     * A list of exception types with their corresponding custom log levels.
     *
     * @var array<class-string<\Throwable>, \Psr\Log\LogLevel::*>
     */
    protected $levels = [
        //
    ];

    /**
     * A list of the exception types that are not reported.
     *
     * @var array<int, class-string<\Throwable>>
     */
    protected $dontReport = [
        //
    ];

    /**
     * A list of the inputs that are never flashed to the session on validation exceptions.
     *
     * @var array<int, string>
     */
    protected $dontFlash = [
        'current_password',
        'password',
        'password_confirmation',
    ];

    /**
     * Register the exception handling callbacks for the application.
     * Modify this method for apis 404 errors
     * @return void
     */
    public function register()
    {
        $this->renderable(function (Exception $e, $request) {
            if ($request->is('api/*')) { // this will run only for apis
                if ($e instanceof NotFoundHttpException) {
                    return General::message('Not found', false, 404);
                }
                if ($e instanceof MethodNotAllowedHttpException) {
                    return General::message('Method is not allowed for the requested route or unauthorized request', false, 405);
                }
                if ($e instanceof UnauthorizedException) {
                    return General::message(General::getMessage($e), false, 401);
                }

                return General::message(General::getMessage($e), false, 401);
            }
        });
    }
}
