<?php

namespace App\Helpers;

use Illuminate\Http\JsonResponse;
use Illuminate\Support\Facades\Log;

class General
{
    const SUCCESS_CODE = 200;
    const ERROR_CODE = 500;
    const UNAUTH_CODE = 401;

    /**
     * @param array $data
     * @param string $message
     * @param bool $status
     * @param int $code
     * @return JsonResponse
     */
    public static function apiResponse(array $data = [], string $message = "", bool $status = true, int $code = self::SUCCESS_CODE, $token = null): JsonResponse
    {
        if ($message == '') {
            trans('Something went wrong.');
        }
        $response = [
            'status' => $status,
            'message' => $message,
        ];
        if ($token) {
            $response['token'] = $token; 
        }

        if (count($data) > 0) {
            $response['data'] = $data;
        }

        return response()->json($response, $code);

    }

    public static function message(string $message = "", bool $status = true, int $code = self::SUCCESS_CODE): JsonResponse
    {
        $response = [
            'status' => $status,
            'message' => $message,
        ];
        return response()->json($response, $code);
    }

    public static function validationMessage($error, array $errorMsg = [], int $code = 200): JsonResponse
    {
        $res = [
            'status' => false,
            'validation' => $error,
        ];
        if(!empty($errorMsg)){
            $res['validation'] = $errorMsg;
        }
        return response()->json($res, self::SUCCESS_CODE);
    }

    /**
     * @param $e
     * @param false $exception
     * @return string
     * @note to show generic message or exception message as
     * well as log the exception
     */
    public static function getMessage($e, bool $exception = true): string
    {
        if ($exception) {
            Log::info($e);
            return addslashes($e->getMessage());
        }

        return 'Some thing went wrong, please try again.';
    }

    public static function uploadFile($request, $fileName = 'image', $file_path = 'image'): string
    {
        if ($request->hasFile($fileName)) {
            $image = $request->{$fileName};
            $filenameWithExt = str_replace(' ', '-', $image->getClientOriginalName());
            $filename = pathinfo($filenameWithExt, PATHINFO_FILENAME);
            $extension = $image->getClientOriginalExtension();
            $fileNameToStore = $filename . '_' . time() . '.' . $extension;
            self::createDir(storage_path('app/public/' . $file_path));
            $image->storeAs('public/' . $file_path, $fileNameToStore);
            return $file_path . '/' . $fileNameToStore;

        }

        return '';
    }

    private static function createDir($path)
    {
        if (!file_exists($path)) {
            mkdir($path, 0777, true);
        }
    }

}
