<?php

use App\Models\Category;
use App\Models\Order;
use App\Models\Role;
use App\Models\RoleModulePermission;
use App\Models\Setting;
use App\Models\TcpUser;
use App\Models\Vehicle;
use Illuminate\Support\Arr;
use Illuminate\Support\Str;

function slugify($value)
{
    $slug = preg_replace('/[\s<>[\]{}|\\^%&\$,\/:;=?@#\'\"]/', '-', mb_strtolower($value));

    // Remove duplicate separators.
    $slug = preg_replace('/-+/', '-', $slug);

    // Trim special characters from the beginning and end of the slug.
    return trim($slug, '!"#$%&\'()*+,-./:;<=>?@[]^_`{|}~');
}

function str_slug($title, $separator = '-', $language = 'en')
{
    return Str::slug($title, $separator, $language);
}

function array_has($array, $keys)
{
    return Arr::has($array, $keys);
}

function active($routes, $class = 'active'): string
{
    if (in_array(request()->route()->getName(), $routes)) {
        return $class;
    }

    return '';
}

function dateFormat($date, $format = 'Y-m-d'): string
{
  return \Illuminate\Support\Carbon::parse($date)->format($format);
}

function excluded($module, $action)
{
    $exist = true;
    $excluded = config('constants.excluded');
    if (isset($excluded[$module])) {
        if (isset($excluded[$module][$action])) {
            $exist = false;
        }
    }
    return $exist;
}

function editIcon()
{
    return '<svg xmlns="http://www.w3.org/2000/svg" width="14" height="14" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-edit-2 mr-50"><path d="M17 3a2.828 2.828 0 1 1 4 4L7.5 20.5 2 22l1.5-5.5L17 3z"></path></svg>';
}

function deleteIcon()
{
    return '<svg xmlns="http://www.w3.org/2000/svg" width="14" height="14" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-trash mr-50"><polyline points="3 6 5 6 21 6"></polyline><path d="M19 6v14a2 2 0 0 1-2 2H7a2 2 0 0 1-2-2V6m3 0V4a2 2 0 0 1 2-2h4a2 2 0 0 1 2 2v2"></path></svg>';
}

function viewIcon()
{
   return '<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-eye"><path d="M1 12s4-8 11-8 11 8 11 8-4 8-11 8-11-8-11-8z"></path><circle cx="12" cy="12" r="3"></circle></svg>';
}

function editableField($column, $value, $route, $id)
{
    if ($value == '') {
        return '<a href="javascript:void(0);" class="editable"><span class="field-value">edit</span></a><input style="width: 200px;" type="text" onchange="fieldEditable(`'.$route.'`, `'.$column.'`, `'.$id.'`, $(this));" class="form-control editable-field d-none" name="'.$column.'">';
    }
    return '<a href="javascript:void(0);" class="editable"><span class="field-value">'.$value.'</span></a><input style="width: 200px;" type="text" onchange="fieldEditable(`'.$route.'`, `'.$column.'`, `'.$id.'`, $(this));" class="form-control editable-field d-none" value="'.$value.'" name="'.$column.'">';
}

function statues(): array
{
    return Order::STATUES;
}

function orderStatuses($order)
{
    $color = 'orange';
    if ($order->status == Order::STATUES['pending']) {
        $color = Order::STATUS_COLORS[Order::STATUES['pending']];
    }
    if ($order->status == Order::STATUES['in-progress']) {
        $color = Order::STATUS_COLORS[Order::STATUES['in-progress']];
    }
    if ($order->status == Order::STATUES['completed']) {
        $color = Order::STATUS_COLORS[Order::STATUES['completed']];
    }
    if ($order->status == Order::STATUES['shipped']) {
        $color = Order::STATUS_COLORS[Order::STATUES['shipped']];
    }
    if ($order->status == Order::STATUES['cancelled']) {
        $color = Order::STATUS_COLORS[Order::STATUES['cancelled']];
    }
    $html = '<select onchange="changeStatues($(this), '.$order->id.');" name="status" class="form-control w-80 badge" style="color: #fff; background-color: '.$color.'">';
    foreach(statues() as $key => $status) {
        $selected = $order->status == $key ? 'selected' : '';
        $html .= '<option value="'.$key.'" '.$selected.'>'.Str::title($status).'</option>';
    }
    $html .= '</select>';

    return $html;
}

function trackingNo($order): string
{
    if($order->status == Order::PENDING || $order->status == Order::INPROGRESS) {
        $html = '<input onchange="sendTrackNo($(this), '.$order->id.');" type="text" class="form-control" style="position:absolute; width:93%;">';
    } else {
        $html = '<input disabled readonly value="'.$order->tracking_no .'" type="text" class="form-control"  style="position:absolute; width:93%;">';
    }
    $html .= '<div style="position: relative;float: right; top: 5px;" class="spinner-border d-none" role="status"><span class="sr-only">Loading...</span></div>';
    return $html;
}

function setting($key, $orignal = false, $default = 'KWD'): string
{
    if (!$orignal) {
        if ($key == 'site_logo') {
            $logo = Setting::where('key', $key)->value('value') ?? '';
            if ($logo != '') {
                return asset('storage/' . $logo);
            }
            return asset('admin/images/logo.png');
        }
    }

    return Setting::where('key', $key)->value('value') ?? '';
}

function settings($columns = [])
{
    if (count($columns) === 0) {
        return Setting::select('value', 'key')->pluck('value', 'key')->toArray();
    }

    $settings =  Setting::select('value', 'key')->pluck('value', 'key')->toArray();

    $values = [];
    foreach ($columns as  $column) {
        if (isset($settings[$column])) {
            $values[$column] = $settings[$column];
        }
    }
    return $values;
}

function hierarchy(Category $category, &$hierarchy)
{
    $category = $category->load('parent');
    $hierarchy[$category?->id] = $category?->id;
    if (isset($category->parent) && $category->parent->parent_id) {
        hierarchy($category->parent, $hierarchy);
        if ($category->parent) {
            $hierarchy[$category->parent->id] = $category->parent?->id;
        }
    } else {
        if ($category->parent) {
            $hierarchy[$category->parent->id] = $category->parent?->id;
        }
    }

    return $hierarchy;
}

function parents($slug = null)
{
    if ($slug) {
        return Category::PARENTS[$slug] ?? '';
    }
    return Category::PARENTS;
}

function lang() {
    return app()->getLocale();
}

function countryISO()
{
    return [
        'KWT' => 'KWT',
        'SAU' => 'SAU',
        'ARE' => 'ARE',
        'QAT' => 'QAT',
        'BHR' => 'BHR',
        'OMN' => 'OMN',
        'JOD' => 'JOD',
        'EGY' => 'EGY',
    ];
}

function colorToHex($color_name)
{
    // standard 147 HTML color names
    $colors  =  array(
        'aliceblue'=>'F0F8FF',
        'antiquewhite'=>'FAEBD7',
        'aqua'=>'00FFFF',
        'aquamarine'=>'7FFFD4',
        'azure'=>'F0FFFF',
        'beige'=>'F5F5DC',
        'beige stripes'=>'F5F5DC',
        'bisque'=>'FFE4C4',
        'black'=>'000000',
        'blanchedalmond '=>'FFEBCD',
        'blue'=>'0000FF',
        'blueviolet'=>'8A2BE2',
        'brown'=>'A52A2A',
        'brownbear'=>'A52A2A',
        'brownowl'=>'A52A2A',
        'burlywood'=>'DEB887',
        'cadetblue'=>'5F9EA0',
        'chartreuse'=>'7FFF00',
        'chocolate'=>'D2691E',
        'coral'=>'FF7F50',
        'cornflowerblue'=>'6495ED',
        'cornsilk'=>'FFF8DC',
        'crimson'=>'DC143C',
        'cyan'=>'00FFFF',
        'darkblue'=>'00008B',
        'darkcyan'=>'008B8B',
        'darkgoldenrod'=>'B8860B',
        'darkgray'=>'A9A9A9',
        'darkgreen'=>'006400',
        'darkgrey'=>'A9A9A9',
        'darkkhaki'=>'BDB76B',
        'darkmagenta'=>'8B008B',
        'darkolivegreen'=>'556B2F',
        'darkorange'=>'FF8C00',
        'darkorchid'=>'9932CC',
        'darkred'=>'8B0000',
        'darksalmon'=>'E9967A',
        'darkseagreen'=>'8FBC8F',
        'darkslateblue'=>'483D8B',
        'darkslategray'=>'2F4F4F',
        'darkslategrey'=>'2F4F4F',
        'darkturquoise'=>'00CED1',
        'darkviolet'=>'9400D3',
        'deeppink'=>'FF1493',
        'deepskyblue'=>'00BFFF',
        'dimgray'=>'696969',
        'dimgrey'=>'696969',
        'dodgerblue'=>'1E90FF',
        'firebrick'=>'B22222',
        'floralwhite'=>'FFFAF0',
        'forestgreen'=>'228B22',
        'fuchsia'=>'FF00FF',
        'gainsboro'=>'DCDCDC',
        'ghostwhite'=>'F8F8FF',
        'gold'=>'FFD700',
        'goldenrod'=>'DAA520',
        'gray'=>'808080',
        'green'=>'008000',
        'greenyellow'=>'ADFF2F',
        'grey'=>'808080',
        'greymonster'=>'808080',
        'greycat'=>'808080',
        'charcoal'=>'36454F',
        'honeydew'=>'F0FFF0',
        'hotpink'=>'FF69B4',
        'indianred'=>'CD5C5C',
        'indigo'=>'4B0082',
        'ivory'=>'FFFFF0',
        'ivery'=>'FFFFF0',
        'cocoa'=>'775554',
        'khaki'=>'F0E68C',
        'lavender'=>'E6E6FA',
        'lavenderblush'=>'FFF0F5',
        'lawngreen'=>'7CFC00',
        'lemonchiffon'=>'FFFACD',
        'lightblue'=>'ADD8E6',
        'lightcoral'=>'F08080',
        'lightcyan'=>'E0FFFF',
        'lightgoldenrodyellow'=>'FAFAD2',
        'lightgray'=>'D3D3D3',
        'lightgreen'=>'90EE90',
        'lightgrey'=>'D3D3D3',
        'lightpink'=>'FFB6C1',
        'lightsalmon'=>'FFA07A',
        'lightseagreen'=>'20B2AA',
        'lightskyblue'=>'87CEFA',
        'lightslategray'=>'778899',
        'lightslategrey'=>'778899',
        'lightsteelblue'=>'B0C4DE',
        'lightyellow'=>'FFFFE0',
        'lime'=>'00FF00',
        'limegreen'=>'32CD32',
        'linen'=>'FAF0E6',
        'magenta'=>'FF00FF',
        'maroon'=>'800000',
        'mediumaquamarine'=>'66CDAA',
        'mediumblue'=>'0000CD',
        'mediumorchid'=>'BA55D3',
        'mediumpurple'=>'9370D0',
        'mediumseagreen'=>'3CB371',
        'mediumslateblue'=>'7B68EE',
        'mediumspringgreen'=>'00FA9A',
        'mediumturquoise'=>'48D1CC',
        'mediumvioletred'=>'C71585',
        'midnightblue'=>'191970',
        'mintcream'=>'F5FFFA',
        'cream'=>'fffdd0',
        'mint'=>'F5FFFA',
        'mistyrose'=>'FFE4E1',
        'moccasin'=>'FFE4B5',
        'navajowhite'=>'FFDEAD',
        'navy'=>'000080',
        'navy stripes'=>'000080',
        'oldlace'=>'FDF5E6',
        'olive'=>'808000',
        'olivedrab'=>'6B8E23',
        'orange'=>'FFA500',
        'orangered'=>'FF4500',
        'orchid'=>'DA70D6',
        'palegoldenrod'=>'EEE8AA',
        'palegreen'=>'98FB98',
        'paleturquoise'=>'AFEEEE',
        'palevioletred'=>'DB7093',
        'papayawhip'=>'FFEFD5',
        'peachpuff'=>'FFDAB9',
        'peru'=>'CD853F',
        'pink'=>'FFC0CB',
        'plum'=>'DDA0DD',
        'powderblue'=>'B0E0E6',
        'purple'=>'800080',
        'red'=>'FF0000',
        'rosybrown'=>'BC8F8F',
        'royalblue'=>'4169E1',
        'saddlebrown'=>'8B4513',
        'salmon'=>'FA8072',
        'sandybrown'=>'F4A460',
        'seagreen'=>'2E8B57',
        'seashell'=>'FFF5EE',
        'sienna'=>'A0522D',
        'silver'=>'C0C0C0',
        'skyblue'=>'87CEEB',
        'sky'=>'87CEEB',
        'slateblue'=>'6A5ACD',
        'slategray'=>'708090',
        'slategrey'=>'708090',
        'snow'=>'FFFAFA',
        'springgreen'=>'00FF7F',
        'steelblue'=>'4682B4',
        'tan'=>'D2B48C',
        'teal'=>'008080',
        'thistle'=>'D8BFD8',
        'tomato'=>'FF6347',
        'turquoise'=>'40E0D0',
        'violet'=>'EE82EE',
        'wheat'=>'F5DEB3',
        'white'=>'FFFFFF',
        'indi pink'=>'FFC0CB',
        'whitesmoke'=>'F5F5F5',
        'yellow'=>'FFFF00',
        'yellowgreen'=>'9ACD32',
        'off-white'=>'36454F',
        'off white'=>'36454F',
        'oatmeal'=>'CDC4AA',
        'melange'=>'c1cf7a',
        'mocha'=>'967969',
    );

    $color_name = strtolower($color_name);
    if (isset($colors[$color_name])) {
        return ('#' . $colors[$color_name]);
    }

    return '';
}

/**
 * @param $editRoute
 * @param $deleteRoute
 * @param $id
 * @return string
 */
function actions($editRoute, $deleteRoute, $id): string
{
    return '<a class="badge badge-primary" style="width:25px;" href="'.$editRoute.'">
                    '.editIcon().'
                </a>&nbsp;
                <a class="badge badge-danger" style="width:25px;" onclick="deleteRecord('.$id.', `'.$deleteRoute.'`);" href="javascript:void(0);">
                    '.deleteIcon().'
                </a>';
}

function imagePreview($image, $extraClass = 'default-img'): string
{
    return '<a style="width: 80px;" class="example-image-link '.$extraClass.'" href="'.$image.'" data-lightbox="example-set">
            <img style="width:80px; height:50px; border: 1px solid #252f4c !important;" class="example-image" src="'.$image.'" alt=""/>
            <i class="fa fa-eye preview-icon" style="display: none;"></i>
            </a>';
}

/**
 * @return string[]
 */
function shifts(): array
{
    return TcpUser::SHIFTS;
}

function categories(): array
{
    return Vehicle::CATEGORIES;
}

