<?php

namespace App\Http\Controllers\Api;

use App\Helpers\General;
use App\Http\Controllers\Controller;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class AuthController extends Controller
{
    /**
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function login(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'email' => 'required|email',
            'password' => 'required|min:8',
        ]);
        if ($validate->fails()) {
            return General::apiResponse([], $validate->errors()->first(), false);
        }

        try {

            $data = $request->only('email', 'password');
            if (!Auth::attempt($data)) {
                return General::apiResponse([], 'The credentials did not match with our record.', false);
            }

            $user = User::select([
                'users.*',
                'tcp_users.shift',
                'tcp_users.cnic',
                'tcp_users.phone',
                'tcps.name as tcp_name',
                'tcps.lat as tcp_lat',
                'tcps.long as tcp_long',
            ])
                ->join('tcp_users', 'tcp_users.user_id', 'users.id')
                ->join('tcps', 'tcps.id', 'tcp_users.tcp_id')
                ->where('email', $request['email'])->firstOrFail();

            $token = $user->createToken('auth_token')->plainTextToken;


            return General::apiResponse($user->toArray(), 'success', true, 200, $token);
        } catch (\Exception $e) {
            return General::apiResponse([], General::getMessage($e), false);
        }
    }
}
