<?php

namespace App\Http\Controllers\Api;

use App\Helpers\General;
use App\Http\Requests\Api\ExitRequest;
use App\Models\Company;
use App\Models\Vehicle;
use App\Models\Visitor;
use App\Http\Controllers\Controller;
use App\Http\Requests\Api\EntryRequest;
use Carbon\Carbon;

class VisitorController extends Controller
{
    /**
     * @param EntryRequest $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function trip(EntryRequest $request)
    {
        try {
            if ($request->hasFile('image')) {
                $request->merge([
                    'visit_image' => General::uploadFile($request, 'image', 'visitor')
                ]);
            }
        	$save = Visitor::create([
        		'tcp_user_id' => auth()->id(),
        		'entry' => $request->entry ?? 0,
        		'exit' => $request->exit ?? 0,
        		'vehicle_id' => $request->vehicle_id,
            	'company_id' => $request->company_id,
           		'category' => $request->category,
            	'visit_image' => $request->visit_image,
            	'description' => $request->description,
            	'time' => Carbon::now(),
        	]);

        	if ($save) {
        		 return General::apiResponse([], 'Visitor entered successfully.');
        	}
        	return General::apiResponse([], 'Visitor failed to entered.', false);

        } catch(\Exception $e) {
        	return General::apiResponse([], General::getMessage($e), false);
        }
    }

    /**
     * @param ExitRequest $request
     * @return \Illuminate\Http\JsonResponse
     * @note: not using
     */
    public function exitTrip(ExitRequest $request)
    {
        try {
            if ($request->hasFile('image')) {
                $request->merge([
                    'exit_image' => General::uploadFile($request, 'image', 'visitor')
                ]);
            }
            $visitor = Visitor::find($request->trip_id);
            if (!$visitor) {
                return General::apiResponse([], 'Visitor not exist.', false);
            }
            $save = $visitor->update([
                'exit_image' => $request->exit_image,
                'exit' => 1,
                'exit_time' => Carbon::now(),
            ]);

            if ($save) {
                return General::apiResponse([], 'Visitor exit successfully.');
            }
            return General::apiResponse([], 'Visitor failed to exit.', false);

        } catch(\Exception $e) {
            return General::apiResponse([], General::getMessage($e), false);
        }
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function todayTrips()
    {
        try {
            $trips = Visitor::select([
                'visitors.*',
                'companies.name as company_name',
                'vehicles.name as vehicle_name',
                'tcps.name as tcp_name',
                'tcps.lat as tcp_lat',
                'tcps.long as tcp_long',
            ])
                ->join('companies', 'companies.id', 'visitors.company_id')
                ->join('vehicles', 'vehicles.id', 'visitors.vehicle_id')
                ->join('tcp_users', 'tcp_users.user_id', 'visitors.tcp_user_id')
                ->join('tcps', 'tcps.id', 'tcp_users.tcp_id')
                ->where('tcp_users.user_id', auth()->id())
                ->whereDate('time', Carbon::now()->format('Y-m-d'))->get();
            return General::apiResponse([
                'trips' => $trips
            ]);
        } catch (\Exception $e) {
            return General::apiResponse([], General::getMessage($e), false);
        }
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function masterData()
    {
        try {
            return General::apiResponse([
                'vehicles' => Vehicle::isActive()->get([
                    'id',
                    'company_id',
                    'registration_no',
                    'category',
                ]),
                'companies' => Company::isActive()->get([
                    'id',
                    'name',
                    'registration_no',
                    'address',
                ]),
            ]);
        } catch (\Exception $e) {
            return General::apiResponse([], General::getMessage($e), false);
        }
    }
}
