<?php

namespace App\Http\Controllers;

use App\Helpers\General;
use App\Models\Tcp;
use App\Models\TcpUser;
use App\Models\Visitor;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class DashboardController extends Controller
{
    public function index()
    {
        return view('dashboard', [
            'total_entries' => $inTcps->total ?? 0,
            'total_exit' => $outTcps->total ?? 0,
            'illegal' => TcpUser::isActive()->count()
        ]);
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function mapVisits()
    {
        try {

            $tcps = Tcp::isActive()->get([
                    'id',
                    'name',
                    'lat',
                    'long',
                ]);

            $TcpMarkers = [];
            $key = 1;
            foreach ($tcps as $tcp) {
                if ($key > 16) {
                    $key= 1;
                }
                $TcpMarkers[] = [$tcp->name ?? '', $tcp->lat ?? '',$tcp->long ?? '', asset('admin/images/markers/'.$key.'.png')];
                $key++;
            }

            return General::apiResponse($TcpMarkers, 'success', true);
        } catch (\Exception $e) {
            return General::apiResponse([], $e->getMessage(), false);
        }
    }

    public function statistics(Request $request)
    {
        $inTcps = Visitor::select([
            DB::raw('COUNT(visitors.id) as total'),
        ])
            ->join('tcp_users', 'tcp_users.user_id', 'visitors.tcp_user_id')
            ->join('tcps', 'tcps.id', 'tcp_users.tcp_id')
            ->whereDate('time', '>=', $request->start_date)
            ->whereDate('time', '<=', $request->end_date)
            ->where('visitors.exit', '0')
            ->first();

        $outTcps = Visitor::select([
            DB::raw('COUNT(visitors.id) as total'),
        ])
            ->join('tcp_users', 'tcp_users.user_id', 'visitors.tcp_user_id')
            ->join('tcps', 'tcps.id', 'tcp_users.tcp_id')
            ->whereDate('time', '>=', $request->start_date)
            ->whereDate('time', '<=', $request->end_date)
            ->where('visitors.exit', '1')
            ->first();

        $totalUsers = TcpUser::isActive()
            ->whereDate('created_at', '>=', $request->start_date)
            ->whereDate('created_at', '<=', $request->end_date)
            ->count();

        $activeTcps = Tcp::isActive()->whereDate('created_at', '>=', $request->start_date)
            ->whereDate('created_at', '<=', $request->end_date)->count();

        return General::apiResponse([
            'in' => $inTcps->total ?? 0,
            'out' => $outTcps->total ?? 0,
            'totalUsers' => $totalUsers,
            'activeTcps' => $activeTcps,
        ]);
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function inTcps(Request $request)
    {
        $inTcps = Visitor::select([
            DB::raw('COUNT(visitors.id) as total'),
            DB::raw("tcps.name")
        ])
            ->join('tcp_users', 'tcp_users.user_id', 'visitors.tcp_user_id')
            ->join('tcps', 'tcps.id', 'tcp_users.tcp_id')
            ->where('visitors.exit', '0')
            ->whereDate('time', '>=', $request->start_date)
            ->whereDate('time', '<=', $request->end_date)
            ->orderByDesc('total')
            ->groupBy('name')->pluck('total', 'name');

        return General::apiResponse([
            'inTcps' => $inTcps->count() > 0 ? $inTcps->values() : [],
            'tcps' => $inTcps->count() > 0 ? $inTcps->keys() : [],
        ]);
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function outTcps(Request $request)
    {
        $outTcps = Visitor::select([
            DB::raw('COUNT(visitors.id) as total'),
            DB::raw("tcps.name")
        ])
            ->join('tcp_users', 'tcp_users.user_id', 'visitors.tcp_user_id')
            ->join('tcps', 'tcps.id', 'tcp_users.tcp_id')
            ->where('visitors.exit', '1')
            ->whereDate('time', '>=', $request->start_date)
            ->whereDate('time', '<=', $request->end_date)
            ->orderByDesc('total')
            ->groupBy('name')->pluck('total', 'name');

        return General::apiResponse([
            'outTcps' => $outTcps->count() > 0 ? $outTcps->values() : [],
            'tcps' => $outTcps->count() > 0 ? $outTcps->keys() : [],
        ]);
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function categoryWiseTcps(Request $request)
    {
        $outTcps = Visitor::select([
            DB::raw('COUNT(visitors.id) as total'),
            DB::raw("vehicles.category")
        ])
            ->join('vehicles', 'visitors.vehicle_id', 'vehicles.id')
            ->where('visitors.exit', '0')
            ->whereDate('time', '>=', $request->start_date)
            ->whereDate('time', '<=', $request->end_date)
            ->orderByDesc('total')
            ->groupBy('category')->pluck('total', 'category');

        return General::apiResponse([
            'categoryTcps' => $outTcps->count() > 0 ? $outTcps->values() : [],
            'categories' => $outTcps->count() > 0 ? $outTcps->keys() : [],
        ]);
    }

}
