<?php

namespace App\Http\Controllers;

use App\Exports\ExportReport;
use App\Models\Tcp;
use Carbon\Carbon;
use App\Models\Visitor;
use Illuminate\Http\Request;
use Maatwebsite\Excel\Facades\Excel;
use Yajra\DataTables\DataTables;

class ReportController extends Controller
{
    public function index()
    {
        $tcps = Tcp::isActive()->get();
        return view('reports.index', compact('tcps'));
    }

    public function datatable(Request $request, $datatable = true)
    {
        $query = Visitor::orderByDesc('time');
        $query = $this->applyFilters($request, $query);
        $query = $this->getJoinData($query);

        if ($datatable) {

            return DataTables::of($query)
                ->addIndexColumn()
                ->addColumn('user', function ($visitor) {
                    $user = $visitor?->user_name ?? 'N/A';
                    return '<span class="badge badge-light-success">' . $user . '</span>';
                })
                ->addColumn('visit_picture', function ($visitor) {
                    return imagePreview($visitor->visit_picture);
                })
                ->addColumn('entry', function ($visitor) {
                    if ($visitor->entry == 1) {
                        return '<span class="badge badge-light-success">Yes</span>';
                    }
                    return '<span class="badge badge-light-warning">No</span>';
                })
                ->addColumn('exit', function ($visitor) {
                    if ($visitor->exit == 1) {
                        return '<span class="badge badge-light-success">Yes</span>';
                    }
                    return '<span class="badge badge-light-warning">No</span>';
                })
                ->addColumn('time', function ($visitor) {
                    return '<span class="badge badge-light-success">'.Carbon::parse($visitor->time)->timezone('Asia/Karachi')->format("Y-m-d h:i A"). '</span>';
                })
                ->addColumn('vehicle', function ($visitor) {
                    $vehicle = $visitor?->vehicle_name ?? 'N/A';
                    return '<span class="badge badge-light-success">' . $vehicle . '</span>';
                })
                ->addColumn('vehicle_registration_no', function ($visitor) {
                    $vehicle_re = $visitor?->vehicle_registration_no ?? '---';
                    return '<span class="badge badge-light-warning">' . $vehicle_re . '</span>';
                })
                ->addColumn('company', function ($visitor) {
                    $company = $visitor?->company_name ?? 'N/A';
                    return '<span class="badge badge-light-success">' . $company . '</span>';
                })
                ->editColumn('actions', function ($visitor) {
                    return '<a class="badge badge-danger" style="width:25px;" onclick="deleteRecord(' . $visitor->id . ', `' . route('visitor.destroy', $visitor) . '`);" href="javascript:void(0);">
                    ' . deleteIcon() . '
                </a>';
                })
                /*Filter custom alias columns*/
                ->filterColumn('user', function ($query, $keyword) {
                    $keywords = trim($keyword);
                    $query->whereRaw("tcp_users.name like ?", ["%{$keywords}%"]);
                })
                ->filterColumn('company', function ($query, $keyword) {
                    $keywords = trim($keyword);
                    $query->whereRaw("companies.name like ?", ["%{$keywords}%"]);
                })
                ->filterColumn('vehicle', function ($query, $keyword) {
                    $keywords = trim($keyword);
                    $query->whereRaw("vehicles.name like ?", ["%{$keywords}%"]);
                })
                ->filterColumn('vehicle_registration_no', function ($query, $keyword) {
                    $keywords = trim($keyword);
                    $query->whereRaw("vehicles.registration_no like ?", ["%{$keywords}%"]);
                })
                /*End Filter custom alias columns*/
                ->setRowId(function ($visitor) {
                    return 'delete-row-' . $visitor->id;
                })
                ->escapeColumns([])
                ->make(true);
        }

        return $query->get();
    }

    public function exportReport(Request $request)
    {
        $visits = $this->datatable($request, false);
        return Excel::download(new ExportReport($visits), time().'-report.xlsx');
    }

    private function getJoinData($query)
    {
        return $query->select([
            'visitors.*',
            'tcp_users.name as user_name',
            'vehicles.name as vehicle_name',
            'vehicles.registration_no as vehicle_registration_no',
            'companies.name as company_name',
            'tcps.name as tcp_name'
        ])
            ->leftJoin('tcp_users', 'tcp_users.user_id', 'visitors.tcp_user_id')
            ->leftJoin('tcps', 'tcp_users.tcp_id', 'tcps.id')
            ->leftJoin('vehicles', 'vehicles.id', 'visitors.vehicle_id')
            ->leftJoin('companies', 'companies.id', 'visitors.company_id');
    }

    private function applyFilters(Request $request, $query)
    {
        if ($request->has('tcp_id') && $request->filled('tcp_id')) {
            $query->where('tcps.id', $request->tcp_id);
        }
        if ($request->has('start_date') && $request->filled('start_date')) {
            $query->whereDate('visitors.time', '>=', $request->start_date);
        }
        if ($request->has('end_date') && $request->filled('end_date')) {
            $query->whereDate('visitors.time', '<=', $request->end_date);
        }
        if ($request->has('visit_type') && $request->filled('visit_type')) {

            if ($request->visit_type == 'in') {
                $query->where('visitors.exit', 0);
            }
            if ($request->visit_type == 'out') {
                $query->where('visitors.exit', 1);
            }
        }

        return $query;
    }
}
