<?php

namespace App\Http\Controllers;

use App\Helpers\General;
use App\Models\Visitor;
use Carbon\Carbon;
use Yajra\DataTables\DataTables;

class VisitorController extends Controller
{
    public function index()
    {
        return view('visitor.index');
    }

    /**
     * @return \Illuminate\Http\JsonResponse
     * @throws \Exception
     */
    public function datatable()
    {
        $query = Visitor::query()->orderByDesc('time');
        return DataTables::of($this->getJoinData($query))
            ->addIndexColumn()
            ->addColumn('user',  function ($visitor) {
                $user = $visitor?->user_name ?? 'N/A';
                return '<span class="badge badge-light-success">'.$user.'</span>';
            })
            ->addColumn('visit_picture',  function ($visitor) {
                return imagePreview($visitor->visit_picture);
            })
            ->addColumn('entry',  function ($visitor) {
                if ($visitor->entry == 1) {
                    return '<span class="badge badge-light-success">Yes</span>';
                }
                return '<span class="badge badge-light-warning">No</span>';
            })
            ->addColumn('exit',  function ($visitor) {
                if ($visitor->exit == 1) {
                    return '<span class="badge badge-light-success">Yes</span>';
                }
                return '<span class="badge badge-light-warning">No</span>';
            })
            ->addColumn('time',  function ($visitor) {
                return '<span class="badge badge-light-success">'.Carbon::parse($visitor->time)->timezone('Asia/Karachi')->format("Y-m-d h:i A"). '</span>';
            })
            ->addColumn('vehicle',  function ($visitor) {
                $vehicle = $visitor?->vehicle_name ?? 'N/A';
                return '<span class="badge badge-light-success">'.$vehicle.'</span>';
            })
            ->addColumn('vehicle_registration_no',  function ($visitor) {
                $vehicle_re = $visitor?->vehicle_registration_no ?? '---';
                return '<span class="badge badge-light-warning">'.$vehicle_re.'</span>';
            })
            ->addColumn('company',  function ($visitor) {
                $company = $visitor?->company_name ?? 'N/A';
                return '<span class="badge badge-light-success">'.$company.'</span>';
            })
            ->editColumn('actions', function($visitor) {
                return '<a class="badge badge-danger" style="width:25px;" onclick="deleteRecord('.$visitor->id.', `'.route('visitor.destroy', $visitor).'`);" href="javascript:void(0);">
                    '.deleteIcon().'
                </a>';
            })
            /*Filter custom alias columns*/
            ->filterColumn('user', function ($query, $keyword) {
                $keywords = trim($keyword);
                $query->whereRaw("tcp_users.name like ?", ["%{$keywords}%"]);
            })
            ->filterColumn('company', function ($query, $keyword) {
                $keywords = trim($keyword);
                $query->whereRaw("companies.name like ?", ["%{$keywords}%"]);
            })
            ->filterColumn('vehicle', function ($query, $keyword) {
                $keywords = trim($keyword);
                $query->whereRaw("vehicles.name like ?", ["%{$keywords}%"]);
            })
            ->filterColumn('vehicle_registration_no', function ($query, $keyword) {
                $keywords = trim($keyword);
                $query->whereRaw("vehicles.registration_no like ?", ["%{$keywords}%"]);
            })
            ->filterColumn('time', function ($query, $keyword) {
                $keywords = trim($keyword);
                $query->whereRaw("visitors.time like ?", ["%{$keywords}%"]);
            })
            /*End Filter custom alias columns*/
            ->setRowId(function ($visitor) {
                return 'delete-row-' . $visitor->id;
            })
            ->escapeColumns([])
            ->make(true);
    }

    private function getJoinData($query)
    {
        return $query->select([
            'visitors.*',
            'tcp_users.name as user_name',
            'vehicles.name as vehicle_name',
            'vehicles.registration_no as vehicle_registration_no',
            'companies.name as company_name',
        ])
            ->leftJoin('tcp_users', 'tcp_users.user_id', 'visitors.tcp_user_id')
            ->leftJoin('vehicles', 'vehicles.id', 'visitors.vehicle_id')
            ->leftJoin('companies', 'companies.id', 'visitors.company_id');
    }

    /**
     * @param Visitor $visitor
     * @return \Illuminate\Http\JsonResponse
     */
    public function destroy(Visitor $visitor)
    {
        try {
            $visitor->delete();
            return response()->json([
                'status' => true,
                'message' => 'Record deleted successfully.'
            ]);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => General::getMessage($e)
            ]);
        }
    }

}
