<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Casts\Attribute;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;

class Visitor extends Model
{
    use HasFactory;

    protected $fillable = [
    	'tcp_user_id',
    	'vehicle_id',
    	'company_id',
    	'entry',
    	'exit',
    	'time',
    	'visit_image',
    	'description',
    	'category',
    	'status'
    ];

    protected $appends = ['visit_picture'];

    /**
     * @param $query
     * @param int $status
     * @return void
     */
    public function scopeIsActive($query, int $status = 1)
    {
        $query->where('status', $status);
    }

    protected function visitPicture(): Attribute
    {
        return Attribute::make(
            get: fn () => $this->visit_image ? asset('storage/'. $this->visit_image) : asset('admin/images/no-image.jpg'),
        );
    }

    /**
     * @return BelongsTo
     */
    public function user(): BelongsTo
    {
        return $this->belongsTo(TcpUser::class, 'tcp_user_id', 'user_id');
    }

    /**
     * @return BelongsTo
     */
    public function vehicle(): BelongsTo
    {
        return $this->belongsTo(Vehicle::class, 'vehicle_id');
    }

    /**
     * @return BelongsTo
     */
    public function company(): BelongsTo
    {
        return $this->belongsTo(Company::class, 'company_id');
    }
}
