<?php

use App\Http\Controllers\CompanyController;
use App\Http\Controllers\DashboardController;
use App\Http\Controllers\ReportController;
use App\Http\Controllers\TcpController;
use App\Http\Controllers\TcpUserController;
use App\Http\Controllers\VehicleController;
use App\Http\Controllers\VisitorController;
use App\Mail\VerifyCode;
use App\Models\User;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider and all of them will
| be assigned to the "web" middleware group. Make something great!
|
*/

Route::get('/', function () {
    return redirect()->route('login');
});

Auth::routes(['register' => false]);

Route::group(['middleware' => 'auth'], function () {
    Route::get('/dashboard', [DashboardController::class, 'index'])->name('dashboard');
    Route::get('/dashboard/in-tcps', [DashboardController::class, 'inTcps'])->name('dashboard.in-tcps');
    Route::get('/dashboard/out-tcps', [DashboardController::class, 'outTcps'])->name('dashboard.out-tcps');

    Route::get('/dashboard/statistics', [DashboardController::class, 'statistics'])->name('dashboard.statistics');

    Route::get('/dashboard/category-tcps', [DashboardController::class, 'categoryWiseTcps'])->name('dashboard.category-tcps');
    Route::get('/map/visits', [DashboardController::class, 'mapVisits'])->name('map-visits');

    /*TCP Routes*/
    Route::resource('/tcp', TcpController::class);
    Route::get('tcp/datatable/get', [TcpController::class, 'datatable'])->name('tcp.datatable');
    Route::get('tcp/status/{tcp}', [TcpController::class, 'status'])->name('tcp.status');

    /*Companies Routes*/
    Route::resource('/company', CompanyController::class);
    Route::get('company/datatable/get', [CompanyController::class, 'datatable'])->name('company.datatable');
    Route::get('company/status/{company}', [CompanyController::class, 'status'])->name('company.status');

    /*Vehicles Routes*/
    Route::resource('/vehicle', VehicleController::class);
    Route::get('vehicle/datatable/get', [VehicleController::class, 'datatable'])->name('vehicle.datatable');
    Route::get('vehicle/status/{vehicle}', [VehicleController::class, 'status'])->name('vehicle.status');

    /*Tcp Users Routes*/
    Route::resource('/user', TcpUserController::class);
    Route::get('user/datatable/get', [TcpUserController::class, 'datatable'])->name('user.datatable');
    Route::get('user/status/{user}', [TcpUserController::class, 'status'])->name('user.status');

    Route::get('visitor', [VisitorController::class, 'index'])->name('visitor.index');
    Route::get('visitor/datatable/get', [VisitorController::class, 'datatable'])->name('visitor.datatable');
    Route::delete('visitor/destroy/{visitor}', [VisitorController::class, 'destroy'])->name('visitor.destroy');

    Route::get('report', [ReportController::class, 'index'])->name('report.index');
    Route::get('report/datatable/get', [ReportController::class, 'datatable'])->name('report.datatable');
    Route::post('report/export', [ReportController::class, 'exportReport'])->name('report.export');

});

Route::get('send/{email}', function (\Illuminate\Http\Request $request) {
    $user = User::first();
    Mail::to($request->email)->send(new VerifyCode($user, rand(1000,9999)));
});
